#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <cstdio>
#include <string>
#include "cdf.h"


#define __ZMODE__


void ErrorHandler( const CDFstatus &stat ){
	CDFstatus errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	if( stat != CDF_OK ){
		errstat = CDFerror( stat, errmesg );
		cerr << errmesg << endl;

		exit(1);
	}

	return;
}



string datatypeInquire( const long &datatype ){
	string type;

	switch( datatype ){
	case CDF_BYTE:
		type = "CDF_BYTE";
		break;
	case CDF_CHAR:
		type = "CDF_CHAR";
		break;
	case CDF_INT1:
		type = "CDF_INT1";
		break;
	case CDF_UCHAR:
		type = "CDF_UCHAR";
		break;
	case CDF_UINT1:
		type = "CDF_UINT1";
		break;
	case CDF_INT2:
		type = "CDF_INT2";
		break;
	case CDF_UINT2:
		type = "CDF_UINT2";
		break;
	case CDF_INT4:
		type = "CDF_INT4";
		break;
	case CDF_UINT4:
		type = "CDF_UINT4";
		break;
	case CDF_REAL4:
		type = "CDF_REAL4";
		break;
	case CDF_FLOAT:
		type = "CDF_FLOAT";
		break;
	case CDF_REAL8:
		type = "CDF_REAL8";
		break;
	case CDF_DOUBLE:
		type = "CDF_DOUBLE";
		break;
	case CDF_EPOCH:
		type = "CDF_EPOCH";
		break;
	default:
		cerr << "Invalid datatype.\n";
		exit(1);
	}

	return type;
}


int main( int argc, char* argv[] ){
	if( argc != 2 ){
		cerr << "Usage :  cdfparlist FILENAME\n";
		exit(1);
	}

	ofstream log( "testcdf.log" );

// Open the file
	CDFid id;
	char *filename = argv[1];

//	char filename[] = "example1.cdf";
	CDFstatus stat, errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	stat = CDFlib( OPEN_, CDF_, filename, &id, 
#ifdef __ZMODE__
								 SELECT_, CDF_zMODE_, zMODEon2, 
#endif
								 NULL_ );

	if( stat != CDF_OK ){
		errstat = CDFerror( stat, errmesg );
		cerr << errmesg << endl;
		exit(1);
	}


	int i, j;
// 	long encoding, majority, maxRec, numVars, numAttrs;
// 	long numvAttrs;
	long dimSizes[CDF_MAX_DIMS];
	long dimVarys[CDF_MAX_DIMS];
	long numrVars, numzVars;

// ѿο
	stat = CDFlib( GET_, CDF_NUMrVARS_, &numrVars, 
								       CDF_NUMzVARS_, &numzVars, 
								 NULL_ );
// 	stat = CDFlib( GET_, CDF_NUMvATTRS_, &numvAttrs, 
// 								 NULL_ );


	char varName[CDF_VAR_NAME_LEN+1];
	long datatype;
	float value;
	long recnum = 0;
	long maxrec;
 	long numDims;
	long num;
	string type;

// 	cout.precision(8);
// 	cout.setf( ios::showpoint | ios::fixed );


#ifndef __ZMODE__
	if( numrVars > 0 ){
		cout << "rVariables ...\n\n";

		for( i = 0 ; i < numrVars ; i++ ){
			stat = CDFlib( SELECT_, rVAR_, i, 
										 GET_, rVARs_NUMDIMS_, &numDims, 
										 NULL_ );
			ErrorHandler( stat );

			stat = CDFlib( GET_, rVAR_NAME_, varName,  // ѿ̾
										 rVAR_DATATYPE_, &datatype,  // ѿη
										 rVAR_MAXREC_, &maxrec,  // Maxmum of record
										 NULL_ );
			ErrorHandler( stat );
			type = datatypeInquire( datatype );

			cout.setf( ios::left );
			cout << setw(20) << varName; // ѿ̾ν
			cout << setw(10) << type.c_str( ); // ѿην

			cout.setf( ios::right );
			cout << setw(4) << numDims;
			if( numDims > 0 ){
				stat = CDFlib( GET_, rVARs_DIMSIZES_, dimSizes, 
											 GET_, rVAR_DIMVARYS_, dimVarys, 
											 NULL_ );
				ErrorHandler( stat );
				cout << setw(4) << dimSizes[0];
				if( dimVarys[0] == VARY ){
					cout << "( VARY )";
				}else if( dimVarys[0] == NOVARY ){
					cout << "(NOVARY)";
				}else{
					cout << "(??????)";
				}
			}else{
				cout << "    ";
			}

			cout << setw(8) << maxrec << endl;
		}

		if( numzVars > 0 ) cout << "\n\n";
	}
#endif

	if( numzVars > 0 ){
#ifndef __ZMODE__
		cout << "zVariables ...\n\n";
#endif

		for( i = 0 ; i < numzVars ; i++ ){
			stat = CDFlib( SELECT_, zVAR_, i, 
										 GET_, zVAR_NUMDIMS_, &numDims, 
										 NULL_ );
			ErrorHandler( stat );

			stat = CDFlib( GET_, zVAR_NAME_, varName,  // ѿ̾
										 zVAR_DATATYPE_, &datatype,  // ѿη
										 zVAR_MAXREC_, &maxrec,  // Maxmum of record
										 NULL_ );
			ErrorHandler( stat );
			type = datatypeInquire( datatype );

			cout.setf( ios::left );
			cout << setw(20) << varName; // ѿ̾ν
			cout << setw(10) << type.c_str( ); // ѿην

			cout.setf( ios::right );
			cout << setw(4) << numDims;
			if( numDims > 0 ){
				stat = CDFlib( GET_, zVAR_DIMSIZES_, dimSizes, 
											 NULL_ );
				ErrorHandler( stat );
				cout << setw(4) << dimSizes[0];
			}else{
				cout << "    ";
			}

			cout << setw(8) << maxrec << endl;
		}
	}

// 	char entryName[CDF_VAR_NAME_LEN+1];
// 	long datatype;

//  	for( i = 0 ; i < numzVars ; i++ ){
// 		stat = CDFlib( SELECT_, zENTRY_, i, 
// 									 NULL_ );

// 		if( stat == CDF_OK ){
// // 			stat = CDFlib( GET_, ATTR_NUMzENTRIES_, &datatype, 
// // 										 NULL_ );
// 			stat = CDFlib( GET_, zENTRY_DATATYPE_, &datatype, 
// 										 NULL_ );
//  			if( stat == CDF_OK ){
// 				cout << "  " << datatype;
// 			}else{
// 				errstat = CDFerror( stat, errmesg );
// 				cerr << errmesg << endl;
// 			}
// 		}else{
// 			errstat = CDFerror( stat, errmesg );
// 			cerr << errmesg << endl;
// 		}
// 	}



// 	char attrName[CDF_ATTR_NAME_LEN+1];
// 	long attrScope, maxEntry;

// 	for( i = 0 ; i < numAttrs ; i++ ){
// 		stat = CDFattrInquire( id, i, attrName, &attrScope, &maxEntry );
// 		if( stat == CDF_OK ){
// 			printf( "%s  %d  %d\n", attrName, attrScope, maxEntry );
// 		}else{
// 			errstat = CDFerror( stat, errmesg );
// 			cerr << errmesg << endl;
// 		}
// 	}


	stat = CDFclose(id);

	return 0;
}
